@@ -1,6 +1,6 @@
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2005 Ross Burton, 2011-2012 Andreas Rönnquist
+ *	Copyright (C) 2005 Ross Burton, 2011-2013 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -40,32 +40,63 @@
 
 #include "error_strings.h"
 
+#include "config.h"
+
 
 /**
  *
  */
-GMainLoop *loop=NULL;
+GMainLoop *loop = NULL;
 
-static gboolean debug=FALSE;
-static gboolean emulate=FALSE;
-static gboolean show_version=FALSE;
+static gboolean debug = FALSE;
+static gboolean emulate = FALSE;
+static gboolean show_version = FALSE;
 
 #ifdef HAVE_GTK3
 	// libwnck Version Information is only availible if you have
 	// libwnck 3.0 or later
-	static gboolean show_wnck_version=FALSE;
+	static gboolean show_wnck_version = FALSE;
 #endif
 
-typedef struct lua_File
+static gchar *script_folder = NULL;
+static gchar *temp_folder = NULL;
+
+GFileMonitor *mon = NULL;
+
+gchar *config_filename = NULL;
+
+/**
+ *
+ */
+static void load_list_of_scripts(WnckScreen *screen, WnckWindow *window,
+											GSList *file_list)
 {
-	gchar *file_name;
-	lua_State *lua_state;
-} _lua_File;
+	GSList *temp_file_list = file_list;
+	// set the window to work on
+	set_current_window(window);
 
-static gchar *script_folder=NULL;
-static gchar *temp_folder=NULL;
+	// for every file in the folder - load the script
+	if (file_window_open_list != NULL) {
 
-static GSList *file_list=NULL;
+		while(temp_file_list) {
+			gchar *filename = (gchar*)temp_file_list->data;
+
+			// is it a LUA file?
+			if (g_str_has_suffix((gchar*)filename,".lua")) {
+
+				// init the script, run it
+				if (!load_script(global_lua_state, filename)) {
+				}
+
+				run_script(global_lua_state);
+
+			}
+			temp_file_list=temp_file_list->next;
+		}
+	}
+	return;
+
+}
 
 
 /**
@@ -73,33 +104,16 @@ static GSList *file_list=NULL;
  */
 static void window_opened_cb(WnckScreen *screen, WnckWindow *window)
 {
-	GSList *temp_file_list=file_list;
-	// set the window to work on
-	set_current_window(window);
+	load_list_of_scripts(screen, window, file_window_open_list);
+}
 
-	// for every file in the folder - load the script
-	if (file_list!=NULL) {
 
-		while(temp_file_list) {
-
-			struct lua_File *lua_file;
-			lua_file=(struct lua_File*)temp_file_list->data;
-
-			// is it a LUA file?
-			if (g_str_has_suffix((gchar*)(lua_file->file_name),".lua")) {
-
-				// init the script, run it
-
-				if (!load_script(lua_file->lua_state,lua_file->file_name)) {
-				}
-
-				run_script(lua_file->lua_state);
-
-			}
-			temp_file_list=temp_file_list->next;
-		}
-	}
-	return;
+/**
+ *
+ */
+static void window_closed_cb(WnckScreen *screen, WnckWindow *window)
+{
+	load_list_of_scripts(screen, window, file_window_close_list);
 }
 
 
@@ -109,11 +123,16 @@ static void window_opened_cb(WnckScreen *screen, WnckWindow *window)
 void init_screens()
 {
 	int i;
-	const int num_screens=gdk_display_get_n_screens(gdk_display_get_default());
-	for (i=0;i<num_screens;i++) {
-		WnckScreen *screen=wnck_screen_get(i);
+	const int num_screens =
+		gdk_display_get_n_screens(gdk_display_get_default());
 
-		g_signal_connect(screen,"window_opened",(GCallback)window_opened_cb,NULL);
+	for (i=0; i<num_screens; i++) {
+		WnckScreen *screen = wnck_screen_get(i);
+
+		g_signal_connect(screen, "window-opened",
+								(GCallback)window_opened_cb, NULL);
+		g_signal_connect(screen, "window-closed",
+								(GCallback)window_closed_cb, NULL);
 	}
 }
 
@@ -123,32 +142,13 @@ void init_screens()
  */
 void devilspie_exit()
 {
-	//done_script();
-	GSList *temp_file_list=file_list;
+	clear_file_lists();
 
-	if (file_list) {
-
-		while(temp_file_list) {
-
-			struct lua_File *lua_file;
-			lua_file=(struct lua_File*)temp_file_list->data;
-
-			if (lua_file) {
-
-				g_free(lua_file->file_name);
-
-				done_script(lua_file->lua_state);
-
-				//lua_file=g_slice_alloc(sizeof(struct lua_File));
-				g_slice_free1(sizeof(struct lua_File),lua_file);
-			}
-
-			temp_file_list=temp_file_list->next;
-		}
-	}
-
-
-	if (temp_folder!=NULL) g_free(temp_folder);
+	if (temp_folder != NULL) g_free(temp_folder);
+	
+	if (mon) g_object_unref(mon);
+	
+	if (config_filename) g_free(config_filename);
 }
 
 
@@ -163,89 +163,45 @@ static void signal_handler(int sig)
 
 	done_script_error_messages();
 
-	if (sig==SIGINT) {
+	if (sig == SIGINT) {
 		exit(EXIT_FAILURE);
 	}
 
 }
 
 
-/**
- * filename_list_sortfunc
- *   function to sort the inserted filenames, to be able to determine
- *   which order files are loaded.
- */
-gint filename_list_sortfunc(gconstpointer a,gconstpointer b)
-{
-	struct lua_File *file1=(struct lua_File *)a;
-	struct lua_File *file2=(struct lua_File *)b;
-
-	return g_ascii_strcasecmp(file1->file_name,file2->file_name);
-}
-
-
-
 /**
  *
  */
-void load_scripts()
+void print_list(GSList *list)
 {
-	GDir *dir;
-	const gchar *current_file;
-
-	// add all the files in the script_folder to the file_list
-	dir=g_dir_open(script_folder,0,NULL);
-	if (!g_file_test(script_folder,G_FILE_TEST_IS_DIR)) {
-
-		printf(_("script_folder isn't a folder."));
-		printf("\n");
-
-		devilspie_exit();
-		exit(EXIT_FAILURE);
-	}
-
-	// a temp list so we dont ruin the start of the list that is stored
-	// in file_list
-	GSList *temp_file_list=file_list;
-
-	int number_of_files=0;
-
-	// add the files in the folder to our linked list
-	while ((current_file=g_dir_read_name(dir))) {
-
-		struct lua_File *lua_file;
-
-		lua_file=g_slice_alloc(sizeof(struct lua_File));
-
-		lua_file->file_name=g_build_path(G_DIR_SEPARATOR_S,
-		                                 script_folder,
-		                                 current_file,
-		                                 NULL);
-
-		// we only bother with *.lua in the folder
-		if (g_str_has_suffix(current_file,".lua")) {
-
-
-			lua_file->lua_state=init_script();
-
-			if (load_script(lua_file->lua_state,lua_file->file_name)!=0) {
-				printf("Error!\n");
+	GSList *temp_list;
+	if (list != NULL) {
+		temp_list = list;
+		
+		while(temp_list) {
+			gchar *file_name = temp_list->data;
+			
+			if (file_name) {
+				if (g_str_has_suffix((gchar*)file_name, ".lua")) {
+					printf("%s\n", (gchar*)file_name);
+				}
 			}
-
-
-			temp_file_list=g_slist_insert_sorted(temp_file_list,
-			                                     (struct lua_File*)lua_file,
-			                                     filename_list_sortfunc);
-
-			number_of_files++;
+			temp_list = temp_list->next;
 		}
 	}
+}
 
-	file_list=temp_file_list;
 
-	g_dir_close(dir);
+/**
+ *
+ */
+void print_script_lists()
+{
+	if (debug)
+		printf("------------\n");
 
-	if (number_of_files==0) {
+	if ((file_window_open_list == NULL) && (file_window_close_list == NULL)) {
 		printf(_("No script files found in the script folder - exiting."));
 		printf("\n\n");
 		exit(EXIT_SUCCESS);
@@ -253,25 +209,67 @@ void load_scripts()
 
 	// If we are running debug mode - print the list of files:
 	if (debug) {
-		printf(_("List of LUA files in folder:"));
+		printf(_("List of LUA files handling \"%s\" events in folder:"),
+			"window_open");
 		printf("\n");
+		if (file_window_open_list)
+			print_list(file_window_open_list);
+	}
 
-		if (file_list!=NULL) {
+	if (debug) {
+		printf(_("List of LUA files handling \"%s\" events in folder:"),
+			"window_close");
+		printf("\n");
+		if (file_window_close_list)
+			print_list(file_window_close_list);
+	}
+}
 
-			while(temp_file_list) {
 
-				struct lua_File *lua_file;
+/**
+ *
+ */
+void folder_changed_callback(GFileMonitor *mon, 
+										GFile *first_file,
+										GFile *second_file, 
+										GFileMonitorEvent event,
+										gpointer user_data)
+{
+	gchar *our_filename = (gchar*)(user_data);
 
-				lua_file=(struct lua_File*)temp_file_list->data;
+	// If a file is created or deleted, we need to check the file lists again
+	if ((event == G_FILE_MONITOR_EVENT_CREATED) || 
+		 (event == G_FILE_MONITOR_EVENT_DELETED)) {
 
-				// is it a LUA file?
-				if (g_str_has_suffix((gchar*)(lua_file->file_name),".lua")) {
-					printf("%s\n",(gchar*)lua_file->file_name);
-				}
+		clear_file_lists();
 
-				// load the script
+		load_config(our_filename);
 
-				temp_file_list=temp_file_list->next;
+		if (debug)
+			printf("Files in folder updated!\n - new lists:\n\n");
+
+		print_script_lists();
+
+		if (debug)
+			printf("-----------\n");
+	}
+
+	// Also monitor if our devilspie2.lua file is changed - since it handles 
+	// which files are window close or window open scripts.
+	if (event == G_FILE_MONITOR_EVENT_CHANGED) {
+		if (first_file) {
+			gchar *short_filename = g_file_get_basename(first_file);
+			
+			if (g_strcmp0(short_filename, "devilspie2.lua")==0) {
+
+				clear_file_lists();
+
+				load_config(our_filename);
+
+				print_script_lists();
+
+				if (debug)
+					printf("----------");
 			}
 		}
 	}
@@ -301,26 +299,27 @@ int main(int argc, char *argv[])
 		{ NULL }
 	};
 
-	GError *error=NULL;
+	GError *error = NULL;
 	GOptionContext *context;
 
 	// Init gettext stuff
-	setlocale(LC_ALL,"");
+	setlocale(LC_ALL, "");
 
-	bindtextdomain(PACKAGE,LOCALEDIR);
-	bind_textdomain_codeset(PACKAGE,"");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset(PACKAGE, "");
 	textdomain(PACKAGE);
 
 	gdk_init(&argc, &argv);
 
-	gchar *devilspie2_description=g_strdup_printf(_("apply rules on windows"));
+	gchar *devilspie2_description =
+		g_strdup_printf(_("apply rules on windows"));
 
-	gchar *full_desc_string=g_strdup_printf("- %s",devilspie2_description);
+	gchar *full_desc_string = g_strdup_printf("- %s", devilspie2_description);
 
 	context=g_option_context_new(full_desc_string);
 	g_option_context_add_main_entries(context,options,NULL);
 	if (!g_option_context_parse(context, &argc, &argv, &error)) {
-		g_print(_("option parsing failed: %s"),error->message);
+		g_print(_("option parsing failed: %s"), error->message);
 		printf("\n");
 		exit(EXIT_FAILURE);
 	}
@@ -329,26 +328,29 @@ int main(int argc, char *argv[])
 	g_free(devilspie2_description);
 
 	// if the folder is NULL, default to ~/.config/devilspie2/
-	if (script_folder==NULL) {
+	if (script_folder == NULL) {
 
-		temp_folder=g_build_path(G_DIR_SEPARATOR_S,g_get_user_config_dir(),"devilspie2",NULL);
+		temp_folder = g_build_path(G_DIR_SEPARATOR_S,
+										g_get_user_config_dir(),
+										"devilspie2",
+										NULL);
 
 		// check if the folder does exist
-		if (!g_file_test(temp_folder,G_FILE_TEST_IS_DIR)) {
+		if (!g_file_test(temp_folder, G_FILE_TEST_IS_DIR)) {
 
 			// - and if it doesn't, create it.
-			if (g_mkdir(temp_folder,0700)!=0) {
+			if (g_mkdir(temp_folder, 0700) != 0) {
 				printf(_("Couldn't create the default folder for devilspie2 scripts."));
 				printf("\n");
 				exit(EXIT_FAILURE);
 			}
 		}
 
-		script_folder=temp_folder;
+		script_folder = temp_folder;
 	}
 
 	if (show_version) {
-		printf("Devilspie2 v%s\n\n",DEVILSPIE2_VERSION);
+		printf("Devilspie2 v%s\n\n", DEVILSPIE2_VERSION);
 		exit(EXIT_SUCCESS);
 	}
 #ifdef HAVE_GTK3
@@ -372,6 +374,21 @@ int main(int argc, char *argv[])
 
 #endif
 
+	if (init_script_error_messages()!=0) {
+		printf(_("Couldn't init script error messages!"));
+		printf("\n");
+		exit(EXIT_FAILURE);
+	}
+
+	config_filename =
+		g_build_filename(script_folder, "devilspie2.lua", NULL);
+
+	if (load_config(config_filename)!=0) {
+
+		devilspie_exit();
+		return EXIT_FAILURE;
+	}
+
 	if (debug) {
 
 		if (emulate) {
@@ -382,23 +399,32 @@ int main(int argc, char *argv[])
 
 		printf("\n\n");
 
-		printf(_("Using scripts from folder: %s"),script_folder);
+		printf(_("Using scripts from folder: %s"), script_folder);
 
 		printf("\n");
 
-		devilspie2_debug=TRUE;
+		devilspie2_debug = TRUE;
 	}
 
 	// Should we only run an emulation (don't modify any windows)
-	if (emulate) devilspie2_emulate=emulate;
-
-	if (init_script_error_messages()!=0) {
-		printf(_("Couldn't init script error messages!"));
+	if (emulate) devilspie2_emulate = emulate;
+	
+	GFile *directory_file;
+	directory_file = g_file_new_for_path(script_folder);
+//	mon = g_file_monitor_directory(directory_file, G_FILE_MONITOR_WATCH_MOUNTS,
+	mon = g_file_monitor_directory(directory_file, G_FILE_MONITOR_NONE,
+												NULL, NULL);
+	if (!mon) {
+		printf(_("Couldn't create directory monitor!"));
 		printf("\n");
-		exit(EXIT_FAILURE);
+		return EXIT_FAILURE;
 	}
+	
+	g_signal_connect(mon, "changed", G_CALLBACK(folder_changed_callback), 
+		(gpointer)(config_filename));
 
-	load_scripts();
+	global_lua_state = init_script();
+	print_script_lists();
 
 	if (debug) printf("------------\n");
 
@@ -417,7 +443,7 @@ int main(int argc, char *argv[])
 
 	init_screens();
 
-	loop=g_main_loop_new(NULL,TRUE);
+	loop=g_main_loop_new(NULL, TRUE);
 	g_main_loop_run(loop);
 
 	return EXIT_SUCCESS;
