@@ -1,6 +1,6 @@
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2011-2012 Andreas Rönnquist
+ *	Copyright (C) 2011-2013 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -152,8 +152,25 @@ int c_set_window_geometry(lua_State *lua)
 		WnckWindow *window=get_current_window();
 
 		if (window) {
+			WnckScreen *screen = wnck_window_get_screen(window);
+			int sw = wnck_screen_get_width(screen);
+			int sh = wnck_screen_get_height(screen);
+
+			int gravity = WNCK_WINDOW_GRAVITY_CURRENT;
+			if (x >= 0 && y >= 0)
+				gravity = WNCK_WINDOW_GRAVITY_NORTHWEST;
+			if (x >= 0 && y < 0)
+				gravity = WNCK_WINDOW_GRAVITY_SOUTHWEST;
+			if (x < 0 && y >= 0)
+				gravity = WNCK_WINDOW_GRAVITY_NORTHEAST;
+			if (x < 0 && y < 0)
+				gravity = WNCK_WINDOW_GRAVITY_SOUTHEAST;
+			if (x < 0)
+				x = sw + x;
+			if (y < 0)
+				y = sh + y;			
 			wnck_window_set_geometry(window,
-			                         WNCK_WINDOW_GRAVITY_CURRENT,
+			                         gravity,
 			                         WNCK_WINDOW_CHANGE_X +
 			                         WNCK_WINDOW_CHANGE_Y +
 			                         WNCK_WINDOW_CHANGE_WIDTH +
@@ -318,14 +335,15 @@ int c_set_window_size(lua_State *lua)
 
 		if (window) {
 
-			my_wnck_error_trap_push();
+			devilspie2_error_trap_push();
 			wnck_window_set_geometry(window,
 				WNCK_WINDOW_GRAVITY_CURRENT,
 				WNCK_WINDOW_CHANGE_WIDTH + WNCK_WINDOW_CHANGE_HEIGHT,
 				-1,-1,x,y);
 
-			if (my_wnck_error_trap_pop()) {
-				gchar *temperror=g_strdup_printf("set_window_size: %s",failed_string);
+			if (devilspie2_error_trap_pop()) {
+				gchar *temperror=
+					g_strdup_printf("set_window_size: %s",failed_string);
 				g_printerr("%s",temperror);
 
 				g_free(temperror);
@@ -405,7 +423,8 @@ int c_get_application_name(lua_State *lua)
 
 	if (window) {
 
-		WnckApplication *application=wnck_window_get_application(get_current_window());
+		WnckApplication *application=
+			wnck_window_get_application(get_current_window());
 		application_name=wnck_application_get_name(application);
 
 	} else {
@@ -585,8 +604,10 @@ int c_undecorate_window(lua_State *lua)
 
 		if (window) {
 			if (!devilspie2_emulate) {
+				
+				gulong xid = wnck_window_get_xid(window);
 
-				if (!undecorate_window(window)) {
+				if (!undecorate_window(xid)) {
 					result=FALSE;
 				}
 			}
@@ -618,7 +639,10 @@ int c_decorate_window(lua_State *lua)
 		if (window) {
 
 			if (!devilspie2_emulate) {
-				if (!decorate_window(window)) {
+				
+				gulong xid = wnck_window_get_xid(window);
+				
+				if (!decorate_window(xid)) {
 					result=FALSE;
 				}
 			}
@@ -647,7 +671,8 @@ int c_set_window_workspace(lua_State *lua)
 	int type=lua_type(lua,1);
 
 	if (type!=LUA_TNUMBER) {
-		luaL_error(lua,"set_window_workspace: %s",number_expected_as_indata_error);
+		luaL_error(lua,"set_window_workspace: %s",
+			number_expected_as_indata_error);
 		return 0;
 	}
 
@@ -1072,7 +1097,8 @@ int c_get_window_is_maximized(lua_State *lua)
 	int top=lua_gettop(lua);
 
 	if (top!=0) {
-		luaL_error(lua,"get_window_is__maximized: %s",no_indata_expected_error);
+		luaL_error(lua,"get_window_is__maximized: %s",
+			no_indata_expected_error);
 		return 0;
 	}
 
@@ -1096,7 +1122,8 @@ int c_get_window_is_maximized_vertically(lua_State *lua)
 	int top=lua_gettop(lua);
 
 	if (top!=0) {
-		luaL_error(lua,"get_window_is_horisontally_maximized: %s",no_indata_expected_error);
+		luaL_error(lua,"get_window_is_horisontally_maximized: %s",
+			no_indata_expected_error);
 		return 0;
 	}
 
@@ -1121,7 +1148,8 @@ int c_get_window_is_maximized_horisontally(lua_State *lua)
 	int top=lua_gettop(lua);
 
 	if (top!=0) {
-		luaL_error(lua,"get_window_is_horisontally_maximized: %s",no_indata_expected_error);
+		luaL_error(lua,"get_window_is_horisontally_maximized: %s",
+			no_indata_expected_error);
 		return 0;
 	}
 
@@ -1154,8 +1182,9 @@ int c_set_window_above(lua_State *lua)
 
 	if (!devilspie2_emulate) {
 
-		my_wnck_change_state(my_wnck_window_get_xscreen(window),
-		                     wnck_window_get_xid(window),
+		Window xid = wnck_window_get_xid(window);
+		devilspie2_change_state(devilspie2_window_get_xscreen(xid),
+		                     xid,
 		                     TRUE,
 		                     my_wnck_atom_get("_NET_WM_STATE_ABOVE"),
 		                     0);
@@ -1181,8 +1210,9 @@ int c_set_window_below(lua_State *lua)
 
 	if (!devilspie2_emulate) {
 
-		my_wnck_change_state(my_wnck_window_get_xscreen(window),
-		                     wnck_window_get_xid(window),
+		Window xid = wnck_window_get_xid(window);
+		devilspie2_change_state(devilspie2_window_get_xscreen(xid),
+		                     xid,
 		                     TRUE,
 		                     my_wnck_atom_get("_NET_WM_STATE_BELOW"),
 		                     0);
@@ -1261,7 +1291,8 @@ int c_get_class_instance_name(lua_State *lua)
 	int top=lua_gettop(lua);
 
 	if (top!=0) {
-		luaL_error(lua,"get_window_class_instance_name: %s",no_indata_expected_error);
+		luaL_error(lua,"get_window_class_instance_name: %s",
+			no_indata_expected_error);
 		return 0;
 	}
 
@@ -1312,7 +1343,8 @@ int c_get_window_property(lua_State *lua)
 	gchar *result=NULL;
 
 	if (window) {
-		result=my_wnck_get_string_property_latin1(wnck_window_get_xid(window),my_wnck_atom_get(value));
+		result=my_wnck_get_string_property_latin1(wnck_window_get_xid(window),
+			my_wnck_atom_get(value));
 	} else {
 		result=g_strdup_printf("NO RESULT");
 	}
@@ -1346,7 +1378,8 @@ int c_get_window_role(lua_State *lua)
 	gchar *result=NULL;
 
 	if (window) {
-		result=my_wnck_get_string_property_latin1(wnck_window_get_xid(window),my_wnck_atom_get("WM_WINDOW_ROLE"));
+		result=my_wnck_get_string_property_latin1(wnck_window_get_xid(window),
+			my_wnck_atom_get("WM_WINDOW_ROLE"));
 	}
 
 	if (result) {
@@ -1406,7 +1439,19 @@ int c_get_window_class(lua_State *lua)
 	gchar *result;
 
 	if (window) {
-		result=g_strdup(wnck_class_group_get_res_class (wnck_window_get_class_group(window)));
+		WnckClassGroup *class_group=wnck_window_get_class_group(window);
+
+#ifdef WNCK_MAJOR_VERSION
+#if WNCK_CHECK_VERSION(3,2,0)
+	gchar *class_name=(char*)wnck_class_group_get_id(class_group);
+#else
+	gchar *class_name=(char*)wnck_class_group_get_res_class (class_group);
+#endif
+#else
+	gchar *class_name=(char*)wnck_class_group_get_res_class (class_group);
+#endif
+
+		result=g_strdup(class_name);
 
 	} else {
 		result=g_strdup("");
@@ -1428,7 +1473,8 @@ int c_set_window_fullscreen(lua_State *lua)
 	int top=lua_gettop(lua);
 
 	if (top!=1) {
-		luaL_error(lua,"set_window_fullscreen: %s", one_indata_expected_error);
+		luaL_error(lua,"set_window_fullscreen: %s",
+			one_indata_expected_error);
 		return 0;
 	}
 
@@ -1437,7 +1483,8 @@ int c_set_window_fullscreen(lua_State *lua)
 	int type=lua_type(lua,1);
 
 	if (type!=LUA_TBOOLEAN) {
-		luaL_error(lua,"set_window_fullscreen: %s",boolean_expected_as_indata_error);
+		luaL_error(lua,"set_window_fullscreen: %s",
+			boolean_expected_as_indata_error);
 		return 0;
 	}
 
@@ -1483,12 +1530,19 @@ int c_set_viewport(lua_State *lua)
 	}
 
 	WnckWindow *window=get_current_window();
+	
+	if (!window) {
+		lua_pushboolean(lua, FALSE);
+		return 1;
+	}
 
 	screen=wnck_window_get_screen(window);
 
 	wnck_window_get_geometry(window, &x, &y, &width, &height);
+	
+	gulong xid = wnck_window_get_xid(window);
 
-	viewport_start=my_wnck_get_viewport_start(window);
+	viewport_start = devilspie2_get_viewport_start(xid);
 	if (viewport_start<0) {
 		g_printerr("set_viewport: %s",could_not_find_current_viewport_error);
 		lua_pushboolean(lua,FALSE);
@@ -1497,12 +1551,12 @@ int c_set_viewport(lua_State *lua)
 
 	x=((num-1) * wnck_screen_get_width(screen)) - viewport_start + x;
 
-	my_wnck_error_trap_push();
+	devilspie2_error_trap_push();
 	XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
 	                  wnck_window_get_xid(window),
 	                  x,y,width,height);
 
-	if (my_wnck_error_trap_pop()) {
+	if (devilspie2_error_trap_pop()) {
 		g_printerr("set_viewport: %s",setting_viewport_failed_error);
 		lua_pushboolean(lua,FALSE);
 		return 1;
@@ -1531,6 +1585,11 @@ int c_center(lua_State *lua)
 	}
 
 	WnckWindow *window=get_current_window();
+	
+	if (!window) {
+		lua_pushboolean(lua, FALSE);
+		return 1;
+	}
 
 	wnck_window_get_geometry(window, NULL, NULL, &window_width, &window_height);
 
@@ -1543,7 +1602,7 @@ int c_center(lua_State *lua)
 	}
 
 	if (workspace==NULL) {
-		g_printerr(_("Cannot get workspace"));
+		g_printerr(_("Could not get workspace"));
 		lua_pushboolean(lua,FALSE);
 		return 1;
 	}
@@ -1554,12 +1613,12 @@ int c_center(lua_State *lua)
 	xoffset = (workspace_width - window_width) / 2;
 	yoffset = (workspace_height - window_height) / 2;
 
-	my_wnck_error_trap_push();
+	devilspie2_error_trap_push();
 	XMoveWindow (gdk_x11_get_default_xdisplay(),
 	             wnck_window_get_xid(window),
 	             xoffset, yoffset);
 
-	if (my_wnck_error_trap_pop()) {
+	if (devilspie2_error_trap_pop()) {
 		g_printerr("center: %s",failed_string);
 		lua_pushboolean(lua,FALSE);
 		return 1;
@@ -1569,3 +1628,66 @@ int c_center(lua_State *lua)
 
 	return 1;
 }
+
+
+/**
+ *
+ */
+int c_set_opacity(lua_State *lua)
+{
+	
+	int top=lua_gettop(lua);
+	//WnckScreen *screen;
+
+	if (top!=1) {
+		luaL_error(lua,"set_opacity: %s", one_indata_expected_error);
+		return 0;
+	}
+
+	int type=lua_type(lua,1);
+	if (type!=LUA_TNUMBER) {
+		luaL_error(lua,"set_opacity: %s",number_expected_as_indata_error);
+		return 0;
+	}
+
+	double value=(double)lua_tonumber(lua,1);
+
+	WnckWindow *window=get_current_window();
+	gulong xid = wnck_window_get_xid(window);
+
+	if (window)
+		my_window_set_opacity(xid, value);
+
+	return 0;
+}
+
+
+/**
+ *
+ */
+int c_set_window_type(lua_State *lua)
+{
+	int top=lua_gettop(lua);
+
+	if (top!=1) {
+		luaL_error(lua,"set_window_type: %s", one_indata_expected_error);
+		return 0;
+	}
+
+	int type=lua_type(lua,1);
+	if (type!=LUA_TSTRING) {
+		luaL_error(lua,"set_window_type: %s",string_expected_as_indata_error);
+		return 0;
+	}
+	
+	gchar *indata=(gchar*)lua_tostring(lua,1);
+
+	WnckWindow *window=get_current_window();
+	
+	if (window) {
+		gulong xid = wnck_window_get_xid(window);
+		my_window_set_window_type(xid, indata);
+	}
+
+	return 0;
+}
