@@ -1,6 +1,6 @@
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2011-2012 Andreas Rönnquist
+ *	Copyright (C) 2011-2013 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -100,4 +100,7 @@ int c_set_viewport(lua_State *lua);
 
 int c_center(lua_State *lua);
 
+int c_set_opacity(lua_State *lua);
+int c_set_window_type(lua_State *lua);
+
 #endif /*__HEADER_SCRIPT_FUNCTIONS_*/
